/** LinuxSSI kernel checkpointer CRTransLib
 * @author John Mehnert-Spahn and Eugen Feller
 *
 * Copyright (c) 2008 Heinrich-Heine-Universitaet Duesseldorf
 * All rights reserved.
 *
 * This file is part of AEM. AEM is part of XtreemOS, a Linux-based Grid
 * Operating System, see <http://www.xtreemos.eu> for more details. The
 * XtreemOS project has been developed with the financial support of the
 * European Commission's IST program under contract #FP6-033576.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the XtreemOS project nor the names of its
 *     partners and/or contributors may be used to endorse or promote
 *     products derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef CRTRANSLIB_LINUXSSI_H
#define CRTRANSLIB_LINUXSSI_H

#define KRG_SERVICES_PATH 		"/proc/kerrighed/services"
#define KRG_CHKPT_PATH			"/var/chkpt"
#define KRG_CHKPT_PATH_LENGTH 		45

#define KERRIGHED_PROC_MAGIC 		0xD1
#define EPM_PROC_BASE 			192

#define CHECKPOINT_PATH_LENGTH 		45
#define KRGCAP_CMD_LENGTH		80

/* Ignore sockets */
#define CKPT_W_UNSUPPORTED_FILE		2
/* Do incremental cp */
#define CKPT_W_INCREMENTAL		3

#define KSYS_XOS_GET_PGRID                 _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 8, \
                                            xos_checkpoint_infos_t)
#define KSYS_XOS_STOP_APP               _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 9, \
                                            xos_checkpoint_infos_t)
#define KSYS_XOS_CHKPT_APP              _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 10, \
                                            xos_checkpoint_infos_t)
#define KSYS_XOS_RESUME_APP_CP             _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 11, \
                                            xos_checkpoint_infos_t)
#define KSYS_XOS_REBUILD_APP            _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 12, \
                                            xos_restart_request_t)
#define KSYS_XOS_RESUME_APP_RST           _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 13, \
                                            xos_restart_request_t)

struct checkpoint_info
{
	long app_id;

	int flags;
	media_t media;

	int chkpt_sn;
	int result;

	int signal;
};

typedef struct xos_checkpoint_info{
	struct checkpoint_info cp_info;
	cp_strategy_t strategy;
	int process_group_reference_id;
	process_group_reference_type_t process_group_reference_type;
}xos_checkpoint_infos_t;

struct restart_request
{
	long app_id;
	int chkpt_sn;
	int flags;
	media_t media;
};

typedef struct xos_restart_request{
	struct restart_request rst_req;
	cp_strategy_t strategy;
	pid_t task_identity_req_pid;
	pid_t task_identity_req_tgid; 
}xos_restart_request_t;

#endif /* CRTRANSLIB_LINUXSSI_H */
