/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file XPamAPIs.h
 * @brief The file is developed for AME, which is
 * call the PAM-awere interface to apply PAM functionalities.
 *
 * The interfaces presented here is specific for AEM's ExecMng.
 * However, it is also a example of PAM-aware interface.
 * */

#ifndef _XPAMAPIS_H
#define _XPAMAPIS_H

#include "xos_policy_rule.h"
#include "xos_utils.h"

#define USE_XPAM
#define PAM 1

/* anqin: use the DEBUG() defined in xos_utils.h */
//#define DEBUG(args...) xos_debug(__FILE__, __LINE__, args)

/** Job information used in AEM */
struct jobInfo{
	char* jobId;
	char* path;
	char* input;
	char* output;
	char* error;
	char** params;
	char** envVar;

	/* added by ICT*/
	unsigned char* jobcert;	/* a buffer of xos-cert holding
				the identity information of job */
	char * checkpointer;

	/* isolation-related parameters: CPU speed, memory size, disk space,
	 * 				network bandwidth */
	double *isolation;
};

struct jobEvent
{
  char *event; //shall be NULL when the event is related to OpenVZ container.
  char *jobId;
  int ppid; //we shall override this one with the container id for OpenVZ
  int pgid;
  int pid; //we shall override this to indicate that the grid job is over; pid = 1
  int gid;
  int exitValue;
  int uid;
};

#define XOS_SIG_CHLD 34

/** job quota */
struct jobQuota {
	enum XPLY_REST restype;
	enum XPLY_QT quo_type;
	float	quo_val;
};

/** PAM-aware interface to execute job
 * @param job the job discriptor
 * @return zero if successful, or non-zero
 * */

int xpamexecvp(struct jobInfo *job);


/** Set job quota
 * @param jobcert the identtiy of job owner. Generally, it is
 *     X509-based certificate
 * @param jobquo the quota of job
 * @return zero if successful, or non-zero
 * */
int setJobQuota(unsigned char *jobcert, struct jobQuota *jobquo);


/** remove job quota
 * @param jobcert the identtiy of job owner. Generally, it is
 *   X509-based certificate
 * @param jobquo the quota of job
 * @return zero if successful, or non-zero
 * */
int removeJobQuota(unsigned char *jobcert, struct jobQuota *jobquo);

#define JOBID_LEN 256
typedef struct msgFromClient{
        int cpid;
        int ppid;
        int exit_status;
        char jobId[JOBID_LEN];
}msgFromClient_t;

struct my_msgbuf {
        long mtype;
        long ctid;
        long portNum; //of the server that watched over this job
        struct msgFromClient msgCln;
};


#endif /*_XPAMAPIS_H*/
